@@ -117,10 +117,12 @@ print.bn = function(x, ...) {
       wcat("  permutations:                         ", format(x$learning$args$B))
     if ("iss" %in% params)
       wcat("  imaginary sample size:                ", format(x$learning$args$iss))
+    if ("iss.mu" %in% params)
+      wcat("  imaginary sample size (normal):       ", format(x$learning$args$iss.mu))
+    if ("iss.w" %in% params)
+      wcat("  imaginary sample size (Wishart):      ", format(x$learning$args$iss.w))
     if ("l" %in% params)
       wcat("  imaginary sample size stepping:       ", format(x$learning$args$l))
-    if ("phi" %in% params)
-      wcat("  phi matrix structure:                 ", x$learning$args$phi)
     if ("k" %in% params)
       wcat("  penalization coefficient:             ", format(x$learning$args$k))
     if ("maxp" %in% params)
