@@ -11,25 +11,28 @@
 #define CHANGED    1
 
 static void scan_graph(int *a, SEXP nodes, int nnodes, short int *collider,
-    int debuglevel);
+    bool debugging);
 static void mark_vstructures(int *a, SEXP nodes, int nnodes, short int *collider,
-    int debuglevel);
+    bool debugging);
 static void unmark_shielded(int *a, SEXP nodes, int nnodes, short int *collider,
-    int debuglevel);
-static int prevent_cycles(int *a, SEXP nodes, int nnodes, short int *collider,
-    int debuglevel);
+    bool debugging);
+static int prevent_cycles(int *a, SEXP nodes, int nnodes, bool debugging);
 static int prevent_additional_vstructures(int *a, SEXP nodes, int nnodes,
-    short int *collider, int debuglevel);
+    bool debugging);
+static int prevent_conflicts(int *a, SEXP nodes, int nnodes, short int *scratch,
+    bool debugging);
+static int prevent_chains(int *a, SEXP nodes, int nnodes, bool debugging);
 static void renormalize_amat(int *a, int nnodes);
 static SEXP amat2vstructs(int *a, SEXP nodes, int nnodes, short int *collider);
 
 /* return the v-structures present in the graph. */
-SEXP vstructures(SEXP arcs, SEXP nodes, SEXP return_arcs, SEXP moral, SEXP debug) {
+SEXP vstructures(SEXP arcs, SEXP nodes, SEXP return_arcs, SEXP including_moral,
+    SEXP debug) {
 
 int i = 0, nnodes = length(nodes);
-int *a = NULL, *ret_arcs = LOGICAL(return_arcs), debuglevel = isTRUE(debug);
-int all_vstructs = isTRUE(moral);
+int *a = NULL, *ret_arcs = LOGICAL(return_arcs);
 short int *collider = NULL;
+bool debugging = isTRUE(debug);
 SEXP amat, result;
 
   /* build the adjacency matrix and dereference it. */
@@ -40,14 +43,14 @@ SEXP amat, result;
   collider = Calloc1D(nnodes, sizeof(short int));
 
   /* STEP 1: scan the graph. */
-  scan_graph(a, nodes, nnodes, collider, debuglevel);
+  scan_graph(a, nodes, nnodes, collider, debugging);
 
   /* STEP 2: all the arcs not part of a v-structure are now undirected. */
-  mark_vstructures(a, nodes, nnodes, collider, debuglevel);
+  mark_vstructures(a, nodes, nnodes, collider, debugging);
 
   /* decide whether or not to keep moral v-structures (i.e. shielded colliders). */
-  if (all_vstructs == 0)
-    unmark_shielded(a, nodes, nnodes, collider, debuglevel);
+  if (!isTRUE(including_moral))
+    unmark_shielded(a, nodes, nnodes, collider, debugging);
 
   /* remove all non-fixed arcs (including compelled ones). */
   for (i = 0; i < nnodes * nnodes; i++)
@@ -131,8 +134,8 @@ SEXP result;
 
 }/*AMAT2VSTRUCTS*/
 
-static int fix_wlbl(int *a, int nnodes, SEXP nodes, SEXP whitelist,
-  SEXP blacklist, int debuglevel) {
+static int fix_arcs(int *a, int nnodes, SEXP nodes, SEXP whitelist,
+  SEXP blacklist, bool debugging) {
 
 int i = 0, j = 0, *w = NULL, *b = NULL;
 SEXP wl, bl;
@@ -176,7 +179,7 @@ SEXP wl, bl;
           if ((a[CMC(i, j, nnodes)] == PRESENT) && (a[CMC(j, i, nnodes)] == ABSENT) &&
               (w[CMC(i, j, nnodes)] == PRESENT)) {
 
-            if (debuglevel > 0)
+            if (debugging)
               Rprintf("  > marking arc %s -> %s as immutable.\n", NODE(i), NODE(j));
 
             a[CMC(i, j, nnodes)] = IMMUTABLE;
@@ -203,7 +206,7 @@ SEXP wl, bl;
           if ((a[CMC(i, j, nnodes)] == PRESENT) && (b[CMC(j, i, nnodes)] == PRESENT) &&
               (b[CMC(i, j, nnodes)] == ABSENT)) {
 
-            if (debuglevel > 0)
+            if (debugging)
               Rprintf("  > marking arc %s -> %s as immutable.\n", NODE(i), NODE(j));
 
             a[CMC(i, j, nnodes)] = IMMUTABLE;
@@ -221,14 +224,15 @@ SEXP wl, bl;
 
   return FALSE;
 
-}/*FIX_WLBL*/
+}/*FIX_ARCS*/
 
-SEXP cpdag(SEXP arcs, SEXP nodes, SEXP moral, SEXP fix, SEXP whitelist,
-    SEXP blacklist, SEXP illegal, SEXP debug) {
+SEXP cpdag(SEXP arcs, SEXP nodes, SEXP moral, SEXP fix, SEXP wlbl,
+    SEXP whitelist, SEXP blacklist, SEXP illegal, SEXP debug) {
 
 int i = 0, changed = 0, nnodes = length(nodes);
-short int *collider = NULL;
-int *a = NULL, debuglevel = isTRUE(debug), all_fixed = FALSE;
+short int *collider = NULL, *scratch = NULL;
+int *a = NULL, all_fixed = FALSE;
+bool debugging = isTRUE(debug);
 SEXP amat;
 
   /* build the adjacency matrix and dereference it. */
@@ -237,39 +241,51 @@ SEXP amat;
 
   /* allocate and initialize a status vector to identify the colliders. */
   collider = Calloc1D(nnodes, sizeof(short int));
+  /* allocate a second scratch vector. */
+  scratch = Calloc1D(nnodes, sizeof(short int));
 
   /* STEP 1a: scan the graph. */
-  scan_graph(a, nodes, nnodes, collider, debuglevel);
+  scan_graph(a, nodes, nnodes, collider, debugging);
 
   /* STEP 1b: fix arcs whose direction is determined by parametric assumptions
    * (the illegal argument works like a blacklist). */
   if (illegal != R_NilValue) {
 
-   if (debuglevel > 0)
-     Rprintf("* setting the directions determined by parametric assumptions (step 1b).\n");
+    if (debugging)
+      Rprintf("* setting the directions determined by parametric assumptions (step 1b).\n");
 
-    fix_wlbl(a, nnodes, nodes, R_NilValue, illegal, debuglevel);
+    fix_arcs(a, nnodes, nodes, R_NilValue, illegal, debugging);
+
+  }/*THEN*/
+
+  /* STEP 1c: fix all directed arcs in the network. */
+  if (isTRUE(fix)) {
+
+    if (debugging)
+      Rprintf("* setting the directions of all directed arcs (step 1c).\n");
+
+    all_fixed = fix_arcs(a, nnodes, nodes, R_NilValue, R_NilValue, debugging);
 
   }/*THEN*/
 
   /* STEP 1c: fix arcs according to the whitelist and the blacklist. */
-  if (isTRUE(fix)) {
+  if (isTRUE(wlbl) && ((whitelist != R_NilValue) || (blacklist != R_NilValue))) {
 
-   if (debuglevel > 0)
-     Rprintf("* setting the directions determined by whitelist and blacklist (step 1c).\n");
+    if (debugging)
+      Rprintf("* setting the directions determined by whitelist and blacklist (step 1c).\n");
 
-    all_fixed = fix_wlbl(a, nnodes, nodes, whitelist, blacklist, debuglevel);
+    all_fixed = fix_arcs(a, nnodes, nodes, whitelist, blacklist, debugging);
 
   }/*THEN*/
 
   if (!all_fixed) {
 
     /* STEP 2: all the arcs not part of a v-structure are now undirected. */
-    mark_vstructures(a, nodes, nnodes, collider, debuglevel);
+    mark_vstructures(a, nodes, nnodes, collider, debugging);
 
     /* decide whether or not to keep moral v-structures (i.e. shielded colliders). */
     if (isTRUE(moral))
-      unmark_shielded(a, nodes, nnodes, collider, debuglevel);
+      unmark_shielded(a, nodes, nnodes, collider, debugging);
 
   }/*THEN*/
 
@@ -277,18 +293,26 @@ SEXP amat;
    * v-structures in the graph. */
   for (i = 0; i < nnodes * nnodes; i++) {
 
-    if (debuglevel > 0)
+    if (debugging)
       Rprintf("* setting the direction of more arcs (step 3, iteration %d).\n", i);
 
-    /* STEP 3a: prevent the creation of cycles. */
-    changed |= prevent_cycles(a, nodes, nnodes, collider, debuglevel);
+    /* STEP 3a: prevent the creation of cycles (Meek Rule 2). */
+    changed |= prevent_cycles(a, nodes, nnodes, debugging);
 
-    /* STEP 3b: prevent the creation of additional v-structures. */
-    changed |= prevent_additional_vstructures(a, nodes, nnodes, collider, debuglevel);
+    /* STEP 3b: prevent the creation of new v-structures (Meek Rule 1). */
+    changed |= prevent_additional_vstructures(a, nodes, nnodes, debugging);
+
+    /* STEP 3c: do not leave arcs undirected if they rule out some directions
+     * of some other undirected arcs (Meek Rule 3). */
+    changed |= prevent_conflicts(a, nodes, nnodes, scratch, debugging);
+
+    /* STEP 3d: avoid being stuck due to long chains mixing directed and
+     * undirected arcs (Meek Rule 4). */
+    changed |= prevent_chains(a, nodes, nnodes, debugging);
 
     if (!changed) {
 
-      if (debuglevel > 0)
+      if (debugging)
         Rprintf("  > the graph is unchanged, stopping.\n");
 
       break;
@@ -296,7 +320,7 @@ SEXP amat;
     }/*THEN*/
 
     /* reset changed for the next iteration. */
-   changed = 0;
+    changed = 0;
 
   }/*FOR*/
 
@@ -308,19 +332,20 @@ SEXP amat;
   UNPROTECT(1);
 
   Free1D(collider);
+  Free1D(scratch);
 
   return amat;
 
 }/*CPDAG*/
 
 static void scan_graph(int *a, SEXP nodes, int nnodes, short int *collider,
-    int debuglevel) {
+    bool debugging) {
 
  int i = 0, j = 0, counter = 0;
 
   /* count the parents of each node (the non-symmetric 1s in the
    * corresponding column of the adjacency matrix). */
-  if (debuglevel > 0)
+  if (debugging)
     Rprintf("* scanning the graph (step 1).\n");
 
   for (j = 0; j < nnodes; j++) {
@@ -341,7 +366,7 @@ static void scan_graph(int *a, SEXP nodes, int nnodes, short int *collider,
 
         collider[j] = 1;
 
-        if (debuglevel > 0)
+        if (debugging)
           Rprintf("  > node %s is the center of a v-structure.\n", NODE(j));
 
         break;
@@ -355,12 +380,12 @@ static void scan_graph(int *a, SEXP nodes, int nnodes, short int *collider,
 }/*SCAN_GRAPH*/
 
 static void mark_vstructures(int *a, SEXP nodes, int nnodes, short int *collider,
-    int debuglevel) {
+    bool debugging) {
 
 int i = 0, j = 0;
 
   /* set arcs belonging to a v-structure as FIXED. */
-  if (debuglevel > 0)
+  if (debugging)
     Rprintf("* marking v-structures (step 2).\n");
 
   for (j = 0; j < nnodes; j++) {
@@ -398,7 +423,7 @@ int i = 0, j = 0;
           /* this is a directed arc, it's part of a v-structure; mark it as FIXED. */
           a[CMC(i, j, nnodes)] = FIXED;
 
-          if (debuglevel > 0)
+          if (debugging)
             Rprintf("  > fixing arc %s -> %s, it's part of a v-structure.\n", NODE(i), NODE(j));
 
         }/*THEN*/
@@ -412,7 +437,7 @@ int i = 0, j = 0;
              * arc in a previous step.*/
             a[CMC(i, j, nnodes)] = a[CMC(j, i, nnodes)] = FIXED;
 
-            if (debuglevel > 0)
+            if (debugging)
               Rprintf("  > fixing arc %s - %s due to conflicting v-structures.\n", NODE(j), NODE(i));
 
           }/*THEN*/
@@ -428,11 +453,11 @@ int i = 0, j = 0;
 }/*MARK_VSTRUCTURES*/
 
 static void unmark_shielded(int *a, SEXP nodes, int nnodes, short int *collider,
-    int debuglevel) {
+    bool debugging) {
 
 int i = 0, j = 0, k = 0, check = FALSE;
 
-  if (debuglevel > 0)
+  if (debugging)
     Rprintf("* removing moral v-structures aka shielded colliders.\n");
 
   for (i = 0; i < nnodes; i++) {
@@ -447,7 +472,7 @@ int i = 0, j = 0, k = 0, check = FALSE;
       if ((a[CMC(j, i, nnodes)] != FIXED) && (a[CMC(j, i, nnodes)] != IMMUTABLE))
         continue;
 
-      if (debuglevel > 0)
+      if (debugging)
         Rprintf("  > considering arc %s -> %s.\n", NODE(j), NODE(i));
 
       /* this is set to TRUE is the arc is part of an unshielded collider. */
@@ -459,7 +484,7 @@ int i = 0, j = 0, k = 0, check = FALSE;
         if (((a[CMC(k, i, nnodes)] != FIXED) && (a[CMC(k, i, nnodes)] != IMMUTABLE)) || (j == k))
           continue;
 
-        if (debuglevel > 0)
+        if (debugging)
           Rprintf("    > considering v-structure %s -> %s <- %s.\n", NODE(j), NODE(i), NODE(k));
 
         /* it's a real v-structure if the parents are not connected or if both
@@ -477,13 +502,13 @@ int i = 0, j = 0, k = 0, check = FALSE;
 
       if (check) {
 
-        if (debuglevel > 0)
+        if (debugging)
           Rprintf("  @ arc %s -> %s is part of a v-structure.\n", NODE(j), NODE(i));
 
       }/*THEN*/
       else {
 
-        if (debuglevel > 0)
+        if (debugging)
           Rprintf("  @ arc %s -> %s is not part of a v-structure.\n", NODE(j), NODE(i));
 
         /* this arc should not be marked as FIXED and should not be directed;
@@ -499,33 +524,31 @@ int i = 0, j = 0, k = 0, check = FALSE;
 
 }/*UNMARK_SHIELDED*/
 
-#define VSTRUCTURE_CHECK(i, j) \
-  for (k = 0, vstructure = FALSE; k < nnodes; k++) { \
-    if ((k == i) || (k == j)) \
-      continue; \
-    if (!((a[CMC(k, j, nnodes)] >= PRESENT) && (a[CMC(j, k, nnodes)] == ABSENT))) \
-      continue; \
-    if ((a[CMC(k, i, nnodes)] >= PRESENT) && (a[CMC(i, k, nnodes)] >= PRESENT)) \
-      continue; \
-    if (!((a[CMC(k, i, nnodes)] >= PRESENT) && (a[CMC(i, k, nnodes)] == ABSENT))) { \
-      vstructure = TRUE; \
-      break; \
-    }/*THEN*/ \
-  }/*FOR*/ \
-  if (vstructure) { \
-    a[CMC(i, j, nnodes)] = FIXED; \
-    a[CMC(j, i, nnodes)] = FIXED; \
-    if (debuglevel > 0) \
-      Rprintf("  > fixing arc %s - %s, one direction introduces new v-structures, the other creates cycles (step 3a).\n", \
-        NODE(j), NODE(i)); \
-    changed = CHANGED; \
-    continue; \
-  }/*THEN*/
+static int vstructures_and_cycles(int *a, int nnodes, int i, int j) {
 
-static int prevent_cycles(int *a, SEXP nodes, int nnodes, short int *collider,
-    int debuglevel) {
+int k = 0;
 
-int i = 0, j = 0, k = 0, vstructure = FALSE, changed = UNCHANGED;
+  for (k = 0; k < nnodes; k++) {
+
+    /* no valid arcs. */
+    if ((k == i) || (k == j))
+      continue;
+
+    /* looking for a v-structure centered in NODE(j), so for NODE(k) -> NODE(j)
+     * and no arc between NODE(i) and NODE(j). */
+    if ((a[CMC(k, j, nnodes)] >= PRESENT) && (a[CMC(j, k, nnodes)] == ABSENT) &&
+        (a[CMC(k, i, nnodes)] == ABSENT) && (a[CMC(i, k, nnodes)] == ABSENT))
+      return TRUE;
+
+  }/*FOR*/
+
+  return FALSE;
+
+}/*VSTRUCTURES_VS_CYCLES*/
+
+static int prevent_cycles(int *a, SEXP nodes, int nnodes, bool debugging) {
+
+int i = 0, j = 0, changed = UNCHANGED;
 int *path = NULL, *scratch = NULL;
 
   /* allocate buffers for c_directed_path(). */
@@ -534,33 +557,32 @@ int *path = NULL, *scratch = NULL;
 
   for (j = 0; j < nnodes; j++) {
 
-    for (i = j + 1; i < nnodes; i++) {
+    for (i = 0; i < nnodes; i++) {
 
       if ((a[CMC(i, j, nnodes)] != PRESENT) || (a[CMC(j, i, nnodes)] != PRESENT))
         continue;
 
       if (c_directed_path(i, j, a, nnodes, nodes, path, scratch, 0)) {
 
-        VSTRUCTURE_CHECK(i, j);
+        if (vstructures_and_cycles(a, nnodes, i, j)) {
 
-        a[CMC(i, j, nnodes)] = FIXED;
-        a[CMC(j, i, nnodes)] = ABSENT;
+          a[CMC(i, j, nnodes)] = FIXED;
+          a[CMC(j, i, nnodes)] = FIXED;
 
-        if (debuglevel)
-          Rprintf("  > fixing arc %s -> %s due to directed path (step 3a).\n", NODE(i), NODE(j));
+          if (debugging)
+            Rprintf("  > fixing arc %s - %s, %s -> %s introduces new v-structures, %s -> %s creates cycles (step 3a).\n", \
+            NODE(i), NODE(j), NODE(j), NODE(i), NODE(i), NODE(j));
 
-        changed = CHANGED;
+        }/*THEN*/
+        else {
 
-      }/*THEN*/
-      else if (c_directed_path(j, i, a, nnodes, nodes, path, scratch, 0)) {
+          a[CMC(i, j, nnodes)] = FIXED;
+          a[CMC(j, i, nnodes)] = ABSENT;
 
-        VSTRUCTURE_CHECK(j, i);
+          if (debugging)
+            Rprintf("  > fixing arc %s -> %s due to directed path (step 3a).\n", NODE(i), NODE(j));
 
-        a[CMC(i, j, nnodes)] = ABSENT;
-        a[CMC(j, i, nnodes)] = FIXED;
-
-        if (debuglevel)
-          Rprintf("  > fixing arc %s -> %s due to directed path (step 3a).\n", NODE(j), NODE(i));
+        }/*THEN*/
 
         changed = CHANGED;
 
@@ -577,13 +599,37 @@ int *path = NULL, *scratch = NULL;
 
 }/*PREVENT_CYCLES*/
 
-static int prevent_additional_vstructures(int *a, SEXP nodes, int nnodes,
-    short int *collider, int debuglevel) {
+short int creates_unshielded_collider(int *a, int nnodes, int j, int i) {
 
-int i = 0, j = 0, k = 0;
+int k = 0;
+
+  /* the target arc is NODE(j) -> NODE(i); so to have an unshielded collider
+   * the graph must contain NODE(k) -> NODE(i) and there must be no arc
+   * between NODE(j) and NODE(K), for at least one NODE(k). */
+  for (k = 0; k < nnodes; k++) {
+
+    if ((k == i) || (k == j))
+      continue;
+
+    if ((a[CMC(k, i, nnodes)] >= PRESENT) && (a[CMC(i, k, nnodes)] == ABSENT) &&
+        (a[CMC(k, j, nnodes)] == ABSENT) && (a[CMC(j, k, nnodes)] == ABSENT))
+
+      return TRUE;
+
+  }/*FOR*/
+
+  return FALSE;
+
+}/*CREATES_UNSHIELDED_COLLIDER*/
+
+static int prevent_additional_vstructures(int *a, SEXP nodes, int nnodes,
+    bool debugging) {
+
+int i = 0, j = 0;
 int *path = NULL, *scratch = NULL;
 short int *has_parent = NULL, *has_neighbour = NULL;
-short int shielded = FALSE, flag = FALSE;
+short int changed = FALSE, unshielded_i = FALSE, unshielded_j = FALSE;
+short int path_to_i = FALSE, path_to_j = FALSE;
 
   has_parent = Calloc1D(nnodes, sizeof(short int));
   has_neighbour = Calloc1D(nnodes, sizeof(short int));
@@ -621,85 +667,118 @@ short int shielded = FALSE, flag = FALSE;
 
     for (i = 0; i < nnodes; i++) {
 
-      if ((a[CMC(i, j, nnodes)] != ABSENT) && (a[CMC(j, i, nnodes)] == PRESENT)) {
+      /* the target arc must be directed. */
+      if (!((a[CMC(i, j, nnodes)] != ABSENT) && (a[CMC(j, i, nnodes)] == PRESENT)))
+        continue;
 
-        if (has_parent[i]) {
+      /* both nodes incident on the arc have parents, be careful not to
+       * introduce v-structures centered on either node. */
+      if (has_parent[i]) {
+
+        unshielded_i = creates_unshielded_collider(a, nnodes, j, i);
+        unshielded_j = creates_unshielded_collider(a, nnodes, i, j);
+        path_to_i = c_directed_path(j, i, a, nnodes, nodes, path, scratch, 0);
+        path_to_j = c_directed_path(i, j, a, nnodes, nodes, path, scratch, 0);
+
+        if (unshielded_j && !unshielded_i) {
+
+          /* if NODE(i) -> NODE(j) introduces an unshielded collider, fix
+           * NODE(j) -> NODE(i) unless it creates cycles. */
+          if (path_to_j) {
+
+            if (debugging)
+              Rprintf("  @ arc %s -> %s introduces cycles, %s -> %s introduces an unshielded collider (step 3b).\n",
+                NODE(i), NODE(j), NODE(j), NODE(i));
+
+            goto failsafe;
+
+          }/*THEN*/
+
+          a[CMC(i, j, nnodes)] = ABSENT;
+          a[CMC(j, i, nnodes)] = FIXED;
+          changed = TRUE;
+
+          if (debugging)
+            Rprintf("  > fixing arc %s -> %s, %s -> %s introduces an unshielded collider (step 3b).\n",
+              NODE(i), NODE(j), NODE(j), NODE(i));
+
+        }/*THEN*/
+        else if (unshielded_j && !unshielded_i) {
+
+          /* if NODE(j) -> NODE(i) introduces an unshielded collider, fix
+           * NODE(i) -> NODE(j) unless it creates cycles. */
+          if (path_to_i) {
+
+            if (debugging)
+              Rprintf("  @ arc %s -> %s introduces cycles, %s -> %s introduces an unshielded collider (step 3b).\n",
+                NODE(j), NODE(i), NODE(i), NODE(j));
+
+            goto failsafe;
+
+          }/*THEN*/
 
+          a[CMC(i, j, nnodes)] = FIXED;
+          a[CMC(j, i, nnodes)] = ABSENT;
+          changed = TRUE;
+
+          if (debugging)
+            Rprintf("  > fixing arc %s -> %s, %s -> %s introduces an unshielded collider (step 3b).\n",
+              NODE(j), NODE(i), NODE(i), NODE(j));
+
+        }/*THEN*/
+        else if (unshielded_i && unshielded_j) {
+
+failsafe:
+
+          /* both directions introduce an unshielded collider, so fix the arc
+           * as undirected. */
           a[CMC(i, j, nnodes)] = FIXED;
           a[CMC(j, i, nnodes)] = FIXED;
+          changed = TRUE;
 
-          flag = TRUE;
-
-          if (debuglevel > 0)
+          if (debugging)
             Rprintf("  > fixing arc %s - %s, both nodes have parents (step 3b).\n", NODE(j), NODE(i));
 
         }/*THEN*/
         else {
 
-          /* NODE(j) has at least one parent, and NODE(i) does not; if all the
-           * parents of NODE(j) are also neighbours of NODE(i) then the arc is
-           * part of a shielded collider and should be left undirected. */
-          for (k = 0, shielded = TRUE; k < nnodes; k++) {
-
-            if ((k == i) || (k == j))
-              continue;
-
-            /* NODE(k) is not a parent of NODE(j), move on. */
-            if (!((a[CMC(k, j, nnodes)] >= PRESENT) && (a[CMC(j, k, nnodes)] == ABSENT)))
-              continue;
-
-            /* NODE(k) is a neighbour of NODE(i), not a parent, so there is no collider. */
-            if ((a[CMC(k, i, nnodes)] >= PRESENT) && (a[CMC(i, k, nnodes)] >= PRESENT))
-              continue;
-
-            /* if NODE(k) is a parent of NODE(j) and of NODE(i), the collider is shielded. */
-            if ((a[CMC(k, i, nnodes)] >= PRESENT) && (a[CMC(i, k, nnodes)] == ABSENT)) {
-
-              shielded = TRUE;
-
-            }/*THEN*/
-            else {
-
-              shielded = FALSE;
-              break;
-
-            }/*ELSE*/
-
-          }/*FOR*/
-
-          if (shielded) {
-
-            if (debuglevel > 0)
-              Rprintf("  > arc %s - %s is not part of an unshielded collider, leave it undirected.\n", NODE(j), NODE(i));
-
-          }/*THEN*/
-          else if (c_directed_path(i, j, a, nnodes, nodes, path, scratch, 0)) {
-
-            a[CMC(i, j, nnodes)] = FIXED;
-            a[CMC(j, i, nnodes)] = FIXED;
-
-            flag = TRUE;
-
-            if (debuglevel > 0)
-              Rprintf("  > fixing arc %s - %s, one direction introduces new v-structures, the other creates cycles (step 3b).\n",
-                NODE(j), NODE(i));
-
-          }/*THEN*/
-          else {
-
-            a[CMC(i, j, nnodes)] = ABSENT;
-            a[CMC(j, i, nnodes)] = FIXED;
-
-            flag = TRUE;
-
-            if (debuglevel > 0)
-              Rprintf("  > fixing arc %s -> %s, prevent v-structure (step 3b).\n", NODE(j), NODE(i));
-
-          }/*THEN*/
+          /* if neither direction introduces an unshielded collider, do no not
+             do anything. */
 
         }/*ELSE*/
 
       }/*THEN*/
+      else {
+
+        if (!creates_unshielded_collider(a, nnodes, i, j)) {
+
+          if (debugging)
+            Rprintf("  > arc %s - %s is not part of an unshielded collider, leave it undirected.\n", NODE(j), NODE(i));
+
+        }/*THEN*/
+        else if (c_directed_path(i, j, a, nnodes, nodes, path, scratch, 0)) {
+
+          a[CMC(i, j, nnodes)] = FIXED;
+          a[CMC(j, i, nnodes)] = FIXED;
+          changed = TRUE;
+
+          if (debugging)
+            Rprintf("  > fixing arc %s - %s, one direction introduces new v-structures, the other creates cycles (step 3b).\n",
+              NODE(j), NODE(i));
+
+        }/*THEN*/
+        else {
+
+          a[CMC(i, j, nnodes)] = ABSENT;
+          a[CMC(j, i, nnodes)] = FIXED;
+          changed = TRUE;
+
+          if (debugging)
+            Rprintf("  > fixing arc %s -> %s, prevent v-structure (step 3b).\n", NODE(j), NODE(i));
+
+        }/*THEN*/
+
+      }/*ELSE*/
 
     }/*FOR*/
 
@@ -710,10 +789,180 @@ short int shielded = FALSE, flag = FALSE;
   Free1D(path);
   Free1D(scratch);
 
-  return flag;
+  return changed;
 
 }/*PREVENT_ADDITIONAL_VSTRUCTURES*/
 
+static int creates_conflicts(int *a, int nnodes, short int *scratch) {
+
+int k = 0, l = 0;
+
+  for (k = 0; k < nnodes; k++) {
+
+    if (!scratch[k])
+      continue;
+
+    for (l = k + 1; l < nnodes; l++) {
+
+      if (!scratch[l])
+        continue;
+
+      /* fix NODE(i) -> NODE(j) if there is such a pair of nodes. */
+      if ((a[CMC(k, l, nnodes)] == ABSENT) && (a[CMC(l, k, nnodes)] == ABSENT))
+        return TRUE;
+
+    }/*FOR*/
+
+  }/*FOR*/
+
+  return FALSE;
+
+}/*CREATES_CONFLICTS*/
+
+static int prevent_conflicts(int *a, SEXP nodes, int nnodes, short int *scratch,
+    bool debugging) {
+
+int i = 0, j = 0, k = 0, changed = FALSE;
+
+  for (i = 0; i < nnodes; i++) {
+
+    for (j = 0; j < nnodes; j++) {
+
+      /* nothing to do, this is not a valid arc. */
+      if (i == j)
+        continue;
+
+      /* nothing to do, this is not an undirected arc. */
+      if (!((a[CMC(i, j, nnodes)] == PRESENT) && (a[CMC(j, i, nnodes)] == PRESENT)))
+        continue;
+
+      /* reset the scratch space. */
+      memset(scratch, '\0', nnodes * sizeof(short int));
+
+      /* look for nodes NODE(K) such that NODE(i) -- NODE(K) -> NODE(j). */
+      for (k = 0; k < nnodes; k++) {
+
+        /* it must be a different node. */
+        if ((k == i) || (k == j))
+          continue;
+
+        if ((a[CMC(i, k, nnodes)] == PRESENT) && (a[CMC(k, i, nnodes)] == PRESENT) &&
+            (a[CMC(k, j, nnodes)] >= PRESENT) && (a[CMC(j, k, nnodes)] == ABSENT)) {
+
+          scratch[k] = TRUE;
+
+          if (debugging)
+            Rprintf("    @ found chain %s - %s -> %s.\n",
+              NODE(i), NODE(k), NODE(j));
+
+        }/*THEN*/
+
+      }/*FOR*/
+
+      /* check if any pair of such nodes are not adjacent; if such a pair exists
+       * fix NODE(i) -> NODE(j). */
+      if (creates_conflicts(a, nnodes, scratch)) {
+
+        a[CMC(i, j, nnodes)] = FIXED;
+        a[CMC(j, i, nnodes)] = ABSENT;
+        changed = TRUE;
+
+        if (debugging)
+          Rprintf("    > fixing %s -> %s because of these chains (step 3c).\n",
+            NODE(i), NODE(j));
+
+        break;
+
+      }/*THEN*/
+
+    }/*FOR*/
+
+  }/*FOR*/
+
+  return changed;
+
+}/*PREVENT_CONFLICTS*/
+
+static int find_chain(int *a, int nnodes, int *elem) {
+
+int i = elem[0], j = elem[1], k = 0, l = 0;
+
+  /* looking for a chain NODE(i) -- NODE(k) -> NODE(l) -> NODE(j). */
+  for (k = 0; k < nnodes; k++) {
+
+    if (!((a[CMC(i, k, nnodes)] >= PRESENT) && (a[CMC(k, i, nnodes)] >= PRESENT)))
+      continue;
+
+    for (l = 0; l < nnodes; l++) {
+
+      if (!((a[CMC(k, l, nnodes)] >= PRESENT) && (a[CMC(l, k, nnodes)] == ABSENT) &&
+            (a[CMC(l, j, nnodes)] >= PRESENT) && (a[CMC(j, l, nnodes)] == ABSENT)))
+        continue;
+
+      /* check whether NODE(i) -- NODE(k) and NODE(i) -- NODE(l) but
+       * NODE(j) is not adjacent to NODE(l). */
+      if ((a[CMC(i, l, nnodes)] >= PRESENT) && (a[CMC(l, i, nnodes)] >= PRESENT) &&
+          (a[CMC(k, j, nnodes)] == ABSENT) && (a[CMC(j, k, nnodes)] == ABSENT)) {
+
+        /* save the nodes in the chain, to use in the debugging output. */
+        elem[0] = k;
+        elem[1] = l;
+
+        return TRUE;
+
+      }/*THEN*/
+
+    }/*FOR*/
+
+  }/*FOR*/
+
+  return FALSE;
+
+}/*FIND_CHAIN*/
+
+static int prevent_chains(int *a, SEXP nodes, int nnodes, bool debugging) {
+
+int i = 0, j = 0, changed = FALSE, chain[2] = {0, 0};
+
+  for (i = 0; i < nnodes; i++) {
+
+    for (j = 0; j < nnodes; j++) {
+
+      /* nothing to do, this is not a valid arc. */
+      if (i == j)
+        continue;
+
+      /* nothing to do, this is not an undirected arc. */
+      if (!((a[CMC(i, j, nnodes)] == PRESENT) && (a[CMC(j, i, nnodes)] == PRESENT)))
+        continue;
+
+      chain[0] = i;
+      chain[1] = j;
+
+      if (find_chain(a, nnodes, chain)) {
+
+        a[CMC(i, j, nnodes)] = FIXED;
+        a[CMC(j, i, nnodes)] = ABSENT;
+        changed = TRUE;
+
+        if (debugging)
+          Rprintf("    @ found chain %s - %s -> %s -> %s with %s - %s and %s - %s, "
+                  "fixing %s -> %s (step 3d).\n",
+            NODE(i), NODE(j), NODE(chain[0]), NODE(chain[1]),
+            NODE(i), NODE(chain[0]), NODE(i), NODE(chain[1]), NODE(i), NODE(j));
+
+        break;
+
+      }/*THEN*/
+
+    }/*FOR*/
+
+  }/*FOR*/
+
+  return changed;
+
+}/*PREVENT_CHAINS*/
+
 static void renormalize_amat(int *a, int nnodes) {
 
   for (int i = 0; i < nnodes * nnodes; i++)
