@@ -1,7 +1,7 @@
 
 /* functions registered to make them visible to .Call() in R. */
 extern SEXP all_equal_bn(SEXP, SEXP);
-extern SEXP allsubs_test(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
+extern SEXP allsubs_test(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP alpha_star(SEXP, SEXP, SEXP);
 extern SEXP amat2arcs(SEXP, SEXP);
 extern SEXP aracne(SEXP, SEXP, SEXP, SEXP, SEXP);
@@ -9,7 +9,7 @@ extern SEXP arcs_cg_assumptions(SEXP, SEXP, SEXP);
 extern SEXP arcs_rbind(SEXP, SEXP, SEXP);
 extern SEXP arcs2amat(SEXP, SEXP);
 extern SEXP arcs2elist(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
-extern SEXP bn_recovery(SEXP, SEXP, SEXP, SEXP, SEXP);
+extern SEXP bn_recovery(SEXP, SEXP, SEXP, SEXP);
 extern SEXP bootstrap_arc_coefficients(SEXP, SEXP);
 extern SEXP bootstrap_reduce(SEXP);
 extern SEXP bootstrap_strength_counters(SEXP, SEXP, SEXP, SEXP);
@@ -25,7 +25,7 @@ extern SEXP chow_liu(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP class_err(SEXP, SEXP);
 extern SEXP configurations(SEXP, SEXP, SEXP);
 extern SEXP count_observed_values(SEXP);
-extern SEXP cpdag(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
+extern SEXP cpdag(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP cpdist_lw(SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP dag2ug(SEXP, SEXP, SEXP);
 extern SEXP data_frame_finite(SEXP);
@@ -75,7 +75,7 @@ extern SEXP per_node_score(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP rbn_master(SEXP, SEXP, SEXP, SEXP);
 extern SEXP reset_test_counter();
 extern SEXP root_nodes(SEXP, SEXP);
-extern SEXP roundrobin_test(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
+extern SEXP roundrobin_test(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP score_cache_fill(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP score_delta(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP shd(SEXP, SEXP, SEXP);
